#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import time


class ConvertMotionCoordinate(object):
    """Convert motion coordinates data.
    Args:
    Example:
    Return:
    Author: 
    """

    def __init__(self):
        super(ConvertMotionCoordinate, self).__init__()
        # ////// Parameters must be configured
        # G-Code file abspath
        self.source_file = r'SiemensContour.txt'
        # The abspath of the output file
        self.target_file = r'motion_coordinates.txt'
        # The type of CPU memory address you want to use
        self.area_type = 'V'
        # Start address of CPU memory used
        self.offset = 1000
        # The coordinates of the Z axis when the pen is up
        self.pen_up_z = 0.0
        # The coordinates of the Z axis when the pen is down
        self.pen_down_z = -8.0
        # ////// parameters does not need to be configured
        self.segment_id = 0
        self.mode = 0
        self.buffer_mode = 0
        self.flag_index = 0
        self.flag_mode = 0
        self.profile_num = 0
        # //////
        self.segments_num = 0
        self.temp_pos_x = 0.0
        # When converting coordinates, the overall X-axis offset
        self.temp_pos_x_offset = 0.0
        self.temp_pos_y = 0.0
        # When converting coordinates, the overall Y-axis offset
        self.temp_pos_y_offset = 0.0
        self.temp_pos_z = 0.0
        self.init_offset = self.offset
        self.segment_length = 13

    def run(self):
        """Convert motion coordinates data.
        Args:
        Example:
        Return:
        """

        start_time = time.time()
        # Do some pre-check
        self.pre_check()
        # Convert source data to DB data
        self.convert_all_lines()
        # Do some final-check
        self.final_check()
        # Print end information
        used_time = time.time() - start_time
        print('Total used time: %ss' % used_time)
        print('End!')

    def pre_check(self):
        """Do some pre-check
        Args:
        Example:
        Return:
        """

        if not os.path.exists(self.source_file):
            raise FileNotFoundError('File %s is not exist.' % self.source_file)
        if os.path.exists(self.target_file):
            os.remove(self.target_file)

    def final_check(self):
        """Do some pre-check
        Args:
        Example:
        Return:
        """

        expect_final_offset = self.init_offset + self.segments_num * self.segment_length - 1
        actual_final_offset = self.offset - 1
        print('\n\nTotal segments number: %s' % self.segments_num)
        print('Expect final offset: %s' % expect_final_offset)
        print('Actual final offset: %s' % actual_final_offset)
        if expect_final_offset != actual_final_offset:
            raise Exception('The actual final offset is not equal to the expect final offset!')

    def get_source_lines(self):
        """Get all the lines of source file
        Args:
        Example:
        Return:
        """

        with open(self.source_file) as f_obj:
            lines = f_obj.readlines()

        return lines

    def convert_all_lines(self):
        """Convert all lines
        Args:
        Example:
        Return:
        """

        # Get source lines
        lines = self.get_source_lines()
        # Convert each line
        for index, line in enumerate(lines):
            if re.search(r'contour', line, re.I):
                continue
            # Move to the special position
            self.convert_single_line(line)
            # After move to the first position of a contour, need to do the pen-down operation
            # After move to the last position of a contour, need to do the pen-up operation
            line_type = self.check_line_type(index, lines)
            if line_type == 'first_line':
                self.convert_single_line(line, 'pen_down')
            if line_type == 'last_line':
                self.convert_single_line(line, 'pen_up')

    @staticmethod
    def check_line_type(index, source_lines):
        """Check whether the line is the first or last line in a contour
        Args:
            index            type(int)        index number of the current line
            source_lines     type(list)       lines data list of source file
        Example:
        Return:
        """

        if index == 0:
            line_type = 'first_line'
        elif index == len(source_lines) - 1:
            line_type = 'last_line'
        else:
            pre_line = source_lines[index - 1]
            next_line = source_lines[index + 1]
            if re.search(r'contour', pre_line, re.I):
                line_type = 'first_line'
            elif re.search(r'contour', next_line, re.I):
                line_type = 'last_line'
            else:
                line_type = 'middle_line'

        return line_type

    def convert_single_line(self, line, special_move=None):
        """Convert all lines
        Args:
            line            type(str)        a line data of source file
            special_move    type(str)        pen_up or pen_down
        Example:
        Return:
        """

        print('Processing line: %s' % line.strip())
        # If the special move is pen_up/pen_down, just need to modify the Z_Pos
        if special_move == 'pen_up':
            self.temp_pos_z = self.pen_up_z
        elif special_move == 'pen_down':
            self.temp_pos_z = self.pen_down_z
        # Parse X_Pos and Y_Pos by source GC information
        # If only has X_Pos or only has Y_Pos in the GC information,
        # the another pos remains the same as the previous segment
        else:
            # Parse X_Pos
            searcher = re.search(r'X\d+(\.\d+)?', line, re.I)
            if searcher:
                temp_pos_x = float(searcher.group()[1:])
                temp_pos_x -= self.temp_pos_x_offset
                temp_pos_x = '%.4f' % temp_pos_x
                self.temp_pos_x = temp_pos_x
            # Parse Y_Pos
            searcher = re.search(r'Y\d+(\.\d+)?', line, re.I)
            if searcher:
                temp_pos_y = float(searcher.group()[1:])
                temp_pos_y -= self.temp_pos_y_offset
                temp_pos_y = '%.4f' % temp_pos_y
                self.temp_pos_y = temp_pos_y

        # Write the DB date for segment id field
        for field in (self.segment_id,):
            area_address = '%sB%s' % (self.area_type, self.offset)
            db_line = '%s %s' % (area_address, field)
            self.offset += 1
            self.write_db_data(db_line)
        # Write the DB date for X_Pos, Y_Pos, Z_Pos field
        for coordinate in (self.temp_pos_x, self.temp_pos_y, self.temp_pos_z):
            area_address = '%sD%s' % (self.area_type, self.offset)
            db_line = '%s %s' % (area_address, coordinate)
            self.offset += 4
            self.write_db_data(db_line)
        # # Write the DB date for mode, buffer_mode field
        # for field in (self.mode, self.buffer_mode):
        #     area_address = '%sB%s' % (self.area_type, self.offset)
        #     db_line = '%s %s' % (area_address, field)
        #     self.offset += 1
        #     self.write_db_data(db_line)
        # # Write the DB date for flag_index field
        # for field in (self.flag_index,):
        #     area_address = '%sW%s' % (self.area_type, self.offset)
        #     db_line = '%s %s' % (area_address, field)
        #     self.offset += 2
        #     self.write_db_data(db_line)
        # # Write the DB date for flag_mode, profile_num field
        # for field in (self.flag_mode, self.profile_num):
        #     area_address = '%sB%s' % (self.area_type, self.offset)
        #     db_line = '%s %s' % (area_address, field)
        #     self.offset += 1
        #     self.write_db_data(db_line)

        # Add a blank line to separate each segment
        self.write_db_data('')
        self.segments_num += 1

    def write_db_data(self, data):
        """Write db data into the target file
        Args:
            data            type(str)        DB data string
        Example:
        Return:
        """

        data += '\n'
        with open(self.target_file, 'a') as f_obj:
            f_obj.write(data)


if __name__ == '__main__':
    ConvertMotionCoordinate().run()
